unit uMain;

{$mode objfpc}{$H+}

interface

uses
  {$IFDEF Windows}
    Windows, ShellApi,
  {$ENDIF}
    Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls,
    Buttons, StdCtrls, Zipper, Types;

type

  TZipInfo = record
    ZipFileName: String;
    FilesInArchive: Integer;
    SizePacked: Int64;
    SizeUnpacked: Int64;
    CompressRate: Integer;
    FileComment: String;
  end;

  FZipInfo = TZipInfo;


  { TfrmMain }

  TfrmMain = class(TForm)
    btnAdd: TBitBtn;
    btnExtract: TBitBtn;
    btnSearch: TBitBtn;
    btnComment: TBitBtn;
    btnInfo: TBitBtn;
    btnOpen: TBitBtn;
    ImageCoolBar: TImageList;
    ImageSmall: TImageList;
    Label1: TLabel;
    Label2: TLabel;
    LV: TListView;
    OpenZipFile: TOpenDialog;
    StatusBar: TStatusBar;
    procedure btnInfoClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure LVCustomDrawItem(Sender: TCustomListView; Item: TListItem;
      State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure LVMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure LVSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
  private
    iItemIndex: Integer;


    {$IFDEF Windows}
      function GetFileIcon(aFile: String; aFolder: boolean = False): HICON;
      function GetFileType(aFile: String): String;
    {$ENDIF}
    function CompressedSize(aEntries: TFullZipFileEntries): Int64;
    function UnCompressedSize(aEntries: TFullZipFileEntries): Int64;
    procedure SetZipInfo(aZipInfo: TZipInfo);
  public
    aZipInfo: FZipInfo;
  end;

var
  frmMain: TfrmMain;


implementation

uses
  uInfo;

{$R *.lfm}


{ TfrmMain }


procedure TfrmMain.btnOpenClick(Sender: TObject);
var
  Z: TUnZipper;
  i: Integer;
  SizeCompressed, SizeUncompressed: Int64;
  Panel1: String = 'Total of %0.n Bytes in %d Files';
  ZipFile: String;
  LI: TListItem;
  IC: TIcon;
begin
  SizeCompressed := 0;
  SizeUncompressed := 0;

  i := 0;
  LV.Clear;
  ZipFile := '';

  OpenZipFile.Filter := 'zip File|*.zip';
  OpenZipFile.InitialDir := ExtractFilePath(Application.ExeName);

  if OpenZipFile.Execute then
    ZipFile := OpenZipFile.FileName;

  if ZipFile = '' then exit;

  //
  z := TUnZipper.Create;
  z.FileName := ZipFile;
  z.Examine; // needed!!!

  SizeUncompressed := UnCompressedSize(z.Entries);
  SizeCompressed   := CompressedSize(z.Entries);

//
aZipInfo.ZipFileName    := ExtractFileName(ZipFile);
aZipInfo.FilesInArchive := z.Entries.Count;
aZipInfo.SizePacked     := SizeCompressed;
aZipInfo.SizeUnpacked   := SizeUncompressed;
aZipInfo.CompressRate   := trunc(aZipInfo.SizePacked / SizeUncompressed * 100);
aZipInfo.FileComment    := z.FileComment;
//

  ImageSmall.Clear;

  LV.BeginUpdate;
  for i := 0 to z.Entries.Count - 1 do
  begin

    // put icons in imagelist
    IC := TIcon.Create;
    if z.Entries.Entries[i].IsDirectory then begin
      IC.Handle := GetFileIcon(' ',true);
    end else begin
      IC.Handle := GetFileIcon(z.Entries.FullEntries[i].ArchiveFileName);
    end;
    ImageSmall.AddIcon( IC );
    IC.Free;

    // fill listview
    LI := LV.Items.Add;
    LI.Caption := z.Entries.FullEntries[i].ArchiveFileName;
    LI.ImageIndex := i;
    LI.SubItems.AddStrings
    ([
      Format('%.0n', [z.Entries.FullEntries[i].Size * 1.0]),
      Format('%.0n', [z.Entries.FullEntries[i].CompressedSize * 1.0]),
      GetFileType( z.Entries.FullEntries[i].ArchiveFileName ),
      FormatDateTime('DD.MM.YYYY hh:nn', z.Entries.FullEntries[i].DateTime),
      z.Entries.FullEntries[i].CRC32.ToHexString
    ]);
  end;
  LV.EndUpdate;

  // set status panel 1
  StatusBar.Panels[1].Text := Format(Panel1, [SizeUncompressed * 1.0, z.Entries.Count]);

  z.Clear;
  z.Free;
end;

procedure TfrmMain.btnInfoClick(Sender: TObject);
begin
  if aZipInfo.ZipFileName <> '' then begin
    frmInfo.ShowModal;
  end else begin
    MessageDlg('Error', 'Please open zipfile first!', mtError, [mbOK], 0);
  end;

end;


procedure TfrmMain.LVCustomDrawItem(Sender: TCustomListView; Item: TListItem;
  State: TCustomDrawState; var DefaultDraw: Boolean);
var
  i: Integer;
  r: TRect;
  ts: TTextStyle;
begin

  // text style pre defs
  ts.Layout      := tlCenter;
  ts.Clipping    := True;
  ts.RightToLeft := False;
  ts.Wordbreak   := False;

  // lets draw
  with Item.ListView.Canvas do begin

    // draw rect for mouse move
    if (Item.Index = iItemIndex) then begin
      r := Item.DisplayRect(drBounds);
      with Item.ListView.Canvas do begin
        Pen.Color := RGBToColor(184,214,251); // frame color
        Brush.Color := RGBToColor(235,243,253);
        RoundRect(r,4,4);
      end;
    end;
    // end

    // draw rect if selected
    if Item.Selected then begin
      r := Item.DisplayRect(drBounds);
      Pen.Color := RGBToColor(125,162,206);
      Brush.Color := RGBToColor(193,219,252);
      RoundRect(r,4,4);
    end;
    // end

    // draw icons
    with LV.SmallImages do begin
    Draw
    (
      Item.ListView.Canvas,
      Item.DisplayRect(drIcon).Left,
      Item.DisplayRect(drIcon).Top + 1,
      Item.ImageIndex,
      DrawingStyle,
      ImageType
    );
    end;
    // end

    // draw mainitems text
    r := Item.DisplayRect(drLabel);
    ts.Alignment := Item.ListView.Column[0].Alignment;
    TextRect(r, r.Left + 2, r.Top, Item.Caption, ts);
    // end

    // draw subitems text
    for i := 0 to Item.SubItems.Count - 1 do begin
      r := Item.DisplayRectSubItem(i + 1, drLabel); // get rect for current subitem
      r := Rect(r.Left + 6, r.Top, r.Right - 6, r.Bottom); // set margins for rect
      ts.Alignment := Item.ListView.Column[i + 1].Alignment; // get textstyles
      TextRect(r, r.Left, r.Top, Item.SubItems[i], ts); // draw subitems
    end;
    // end

  end;
  // end with

  DefaultDraw := False;

end;


procedure TfrmMain.LVMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  TmpLI: TListItem;
  TmpLV: TListView absolute Sender;
begin
  TmpLI := TmpLV.GetItemAt(X, Y);

  if TmpLI <> nil then begin
    if iItemIndex <> TmpLI.Index then begin
      iItemIndex := TmpLI.Index;
      TmpLV.Invalidate;
    end;
  end;
end;

procedure TfrmMain.LVSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  i: Integer;
  FileSize: Int64;
  RowSize: Int64;
  vTextIsAre: String;
  vTextFiles: String;
  Panel0: String = 'Selected %s %d %s with %0.n Bytes';
begin
  FileSize   := 0;
  RowSize    := 0;
  vTextIsAre := '';
  vTextFiles := '';

  // selected filesize, if more then 1, then add them together
  for i:=0 to LV.Items.Count - 1 do
  begin
    if LV.Items[i].Selected then
    begin
      RowSize := StringReplace(LV.Items[i].SubItems.Strings[0], '.', '', [rfReplaceAll]).ToInt64;
      Inc(FileSize, RowSize);
    end;
  end;

  // Text for Statusbar
  if LV.SelCount > 1 then begin
    vTextFiles := 'Files';
    vTextIsAre := 'are';
  end else begin
    vTextFiles := 'File';
    vTextIsAre := 'is';
  end;

  // Statusbar
  if LV.SelCount > 0 then begin
    StatusBar.Panels[0].Text :=
      Format(Panel0, [vTextIsAre, LV.SelCount, vTextFiles, FileSize * 1.0]);
  end;

end;

function TfrmMain.UnCompressedSize(aEntries: TFullZipFileEntries): Int64;
var
  i: Integer;
  Size: Int64;
begin
  Size := 0;
  for i := 0 to aEntries.Count - 1 do
  begin
    Inc(Size, aEntries.Entries[i].Size);
  end;
  Result := Size;
end;

function TfrmMain.CompressedSize(aEntries: TFullZipFileEntries): Int64;
var
  i: Integer;
  Size: Int64;
begin
  Size := 0;
  for i := 0 to aEntries.Count - 1 do
  begin
    Inc(Size, aEntries.FullEntries[i].CompressedSize);
  end;
  Result := Size;
end;

procedure TfrmMain.SetZipInfo(aZipInfo: TZipInfo);
begin

end;

{$IFDEF Windows}
// get filetype description from Windows
function TfrmMain.GetFileType(aFile: String): String;
var
  SFI: SHFileInfo;
  Flags: UInt;
begin
  Flags := SHGFI_USEFILEATTRIBUTES or SHGFI_TYPENAME;

  if SHGetFileInfo(PChar(aFile), 0, SFI{%H-}, SizeOf(SFI), Flags) <> 0 then begin
    Result := SFI.szTypeName;
  end else begin
    Result := '';
  end;
end;

// get the fileicon from Windows
function TfrmMain.GetFileIcon(aFile: String; aFolder: boolean = False): HICON;
var
  SFI: SHFileInfo;
  Flags: UInt;
  FileAtt: DWord;
begin
  Flags := SHGFI_USEFILEATTRIBUTES or SHGFI_ICON or SHGFI_SMALLICON or SHGFI_OPENICON;

  if aFolder then begin
    FileAtt := FILE_ATTRIBUTE_DIRECTORY;
  end else begin
    FileAtt := 0;
  end;

  if SHGetFileInfo(PChar(aFile), FileAtt, SFI{%H-}, SizeOf(SFI), Flags ) <> 0 then begin
    Result := SFI.hIcon;
  end else begin
    Result := 0;
  end;
end;
{$ENDIF}


end.

